------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

local ctRelativeFrame = module.ctRelativeFrame;
local appliedOptions;

--------------------------------------------
-- Class Bar (Shapeshift Bar)

local function addon_UpdateOrientation(self, orientation)
	-- Update the frame's orientation.
	-- self == class bar object
	-- orientation == "ACROSS" or "DOWN"

	if (InCombatLockdown()) then
		return;
	end

	orientation = orientation or "ACROSS";

	ShapeshiftButton1:ClearAllPoints();
	ShapeshiftButton1:SetPoint("BOTTOMLEFT", self.frame);

	local obj;
	local spacing = 8;
	for i = 2, 10, 1 do
		obj = _G["ShapeshiftButton"..i];
		obj:ClearAllPoints();
		if ( orientation == "ACROSS" ) then
			obj:SetPoint("LEFT", _G["ShapeshiftButton" .. (i-1)], "RIGHT", spacing, 0);
		else
			obj:SetPoint("TOP", _G["ShapeshiftButton" .. (i-1)], "BOTTOM", 0, -spacing);
		end
	end
end

local function addon_HideTextures(self)
	if (self.isDisabled) then
		return;
	end

	local frames = self.textures;
	if (frames) then
		for i, frame in ipairs(frames) do
			frame:Hide();
			frame:SetVertexColor(1, 1, 1, 0);
		end
	end
end

local function addon_Update(self)
	-- Update the frame
	-- self == class bar object

	addon_HideTextures(self);

	if (not InCombatLockdown()) then
		ShapeshiftBarFrame:EnableMouse(0);

		local obj1 = ShapeshiftButton1;
		local obj2 = ShapeshiftButton10;

		self.helperFrame:ClearAllPoints();
		self.helperFrame:SetPoint("TOPLEFT", obj1);
		self.helperFrame:SetPoint("BOTTOMRIGHT", obj2);

		addon_UpdateOrientation(self, self.orientation);
	end
end

local function addon_Hooked_ShapeshiftBar_Update()
	-- (hooksecurefunc of ShapeshiftBar_Update in BonusActionBarFrame.lua)

	if (module.ctClassBar.isDisabled) then
		return;
	end

	-- Blizzard's function re-anchors ShapeshiftButton1 when the player has only 1 shapeshift form.
	-- That stops us from being able to move the class bar.
	-- We have to undo their anchor and re-establish our own.

	addon_Update(module.ctClassBar);
end

local function addon_Disable(self)
	-- Adjust textures (based on ShapeshiftBar_Update in BonusActionBarFrame.lua)
	local numForms = GetNumShapeshiftForms();
	if ( numForms > 0 ) then
		if ( numForms == 1 ) then
			ShapeshiftBarMiddle:Hide();
			ShapeshiftBarRight:ClearAllPoints();
			ShapeshiftBarRight:SetPoint("LEFT", "ShapeshiftBarLeft", "LEFT", 12, 0);
		elseif ( numForms == 2 ) then
			ShapeshiftBarMiddle:Hide();
			ShapeshiftBarRight:ClearAllPoints();
			ShapeshiftBarRight:SetPoint("LEFT", "ShapeshiftBarLeft", "RIGHT", 0, 0);
		else
			ShapeshiftBarMiddle:Show();
			ShapeshiftBarMiddle:ClearAllPoints();
			ShapeshiftBarMiddle:SetPoint("LEFT", "ShapeshiftBarLeft", "RIGHT", 0, 0);
			ShapeshiftBarMiddle:SetWidth(37 * (numForms-2));
			ShapeshiftBarMiddle:SetTexCoord(0, numForms-2, 0, 1);
			ShapeshiftBarRight:ClearAllPoints();
			ShapeshiftBarRight:SetPoint("LEFT", "ShapeshiftBarMiddle", "RIGHT", 0, 0);
		end
	end

	if (CT_BarMod and CT_BarMod.CT_BarMod_Shift_Shapeshift_UpdatePositions) then
		CT_BarMod.CT_BarMod_Shift_Shapeshift_UpdatePositions();
	end

	-- Call Blizzard's function which may adjust the position
	-- of the bar, and hide/show ShapeShift textures.
	UIParent_ManageFramePositions();
end

local function addon_Init(self)
	-- Initialization
	-- self == object

	appliedOptions = module.appliedOptions;

	module.ctClassBar = self;

	local frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_GuideFrame");
	self.helperFrame = frame;

	hooksecurefunc("ShapeshiftBar_Update", addon_Hooked_ShapeshiftBar_Update);

	return true;
end

local function addon_Register()
	module:registerAddon(
		"Class Bar",  -- option name
		"ClassBar",  -- used in frame names
		"Class Bar",  -- shown in options window & tooltips
		"Class Bar",  -- title for horizontal orientation
		"Class",  -- title for vertical orientation
		{ "BOTTOMLEFT", ctRelativeFrame, "BOTTOM", -484, 107 },
		{ -- settings
			orientation = "ACROSS",
			-- These classes have this bar and therefore will have this bar enabled by default.
			["perClass"] = {
				["DEATHKNIGHT"] = true, -- presences
				["DRUID"] = true,  -- forms
				["HUNTER"] = true,  -- aspects
				["PALADIN"] = true,  -- auras
				["PRIEST"] = true,  -- shadowform
				["ROGUE"] = true,  -- stealth
				["WARRIOR"] = true,  -- stances
			},
		},
		addon_Init,
		nil,  -- no post init function
		addon_Update,
		addon_UpdateOrientation,
		nil,  -- no enable func
		addon_Disable,
		"helperFrame",
		ShapeshiftButton1,
		ShapeshiftButton2,
		ShapeshiftButton3,
		ShapeshiftButton4,
		ShapeshiftButton5,
		ShapeshiftButton6,
		ShapeshiftButton7,
		ShapeshiftButton8,
		ShapeshiftButton9,
		ShapeshiftButton10,
		"",  -- Empty string indicates start of textures
		ShapeshiftBarLeft,
		ShapeshiftBarMiddle,
		ShapeshiftBarRight
	);
end

module.loadedAddons["Class Bar"] = addon_Register;
