------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

local ctRelativeFrame = module.ctRelativeFrame;
local appliedOptions;
local setpoint;
local setparent;

--------------------------------------------
-- Menu Bar

local function addon_UpdateOrientationNoVehicle(self, orientation)
	-- Anchor the frames when the player does not have a vehicle ui.
	-- self == menu bar object
	-- orienation == "ACROSS" or "DOWN"

	local frames = self.frames;
	local obj;

	orientation = orientation or "ACROSS";

	for i = 2, #frames do
		obj = frames[i];
		setparent(obj, self.frame);
		obj:EnableMouse(true);
		obj:ClearAllPoints();
	end

	for i = 3, #frames do
		obj = frames[i];
		if (orientation == "ACROSS") then
			setpoint(obj, "LEFT", frames[i-1], "RIGHT", -3, 0);
		else
			-- Using a large Y value (23) since the buttons look shorter than they are.
			-- See also the talent and achievement hooked functions for the same Y value.
			setpoint(obj, "TOP", frames[i-1], "BOTTOM", 0, 23);
		end
	end

	obj = frames[2];  -- CharacterMicroButton
	setpoint(obj, "BOTTOMLEFT", self.frame, 0, -2);
end

local function addon_UpdateOrientationInVehicle(self, orientation)
	-- Anchor the frames when the player has a vehicle UI
	-- self == menu bar object
	-- orienation == "ACROSS" or "DOWN"

	local skinName = module.vehicleSkinName;
	if (not skinName) then
		return;
	end

	local bar = VehicleMenuBar;
	local parent = VehicleMenuBarArtFrame;
	local frames = self.frames;

	for i = 2, #frames do
		obj = frames[i];
		setparent(obj, parent);
		obj:EnableMouse(true);
		obj:ClearAllPoints();
	end

	for i = 3, #frames do
		obj = frames[i];
		if (orientation == "ACROSS") then
			setpoint(obj, "LEFT", frames[i-1], "RIGHT", -3, 0);
		else
			-- Using a large Y value (23) since the buttons look shorter than they are.
			-- See also the talent and achievement hooked functions for the same Y value.
			setpoint(obj, "TOP", frames[i-1], "BOTTOM", 0, 23);
		end
	end

	local obj1 = frames[2];  -- CharacterMicroButton
	local obj2 = frames[7];  -- GuildMicroButton

	-- The numbers and points are from VehicleMenuBar_MoveMicroButtons() in VehicleMenuBar.lua.

	if (skinName == "Natural") then
		setpoint(obj1, "BOTTOMLEFT", bar, "BOTTOMRIGHT", -365, 41);
		setpoint(obj2, "TOPLEFT", obj1, "BOTTOMLEFT", 0, 20);		
	else -- if (skinName == "Mechanical") then
		setpoint(obj1, "BOTTOMLEFT", bar, "BOTTOMRIGHT", -340, 41);
		setpoint(obj2, "TOPLEFT", obj1, "BOTTOMLEFT", 0, 20);		
	end
end

local function addon_UpdateOrientationBonusBar(self, orientation)
	-- Anchor the frames when the player has a special bonus bar
	-- self == menu bar object
	-- orienation == "ACROSS" or "DOWN"

	local barType = GetBonusBarOverrideBarType() or "default";
	if (barType == "default") then
		return;
	end
	if (not BonusActionBarFrame.microBarX) then
		return;
	end

	local bar = BonusActionBarFrame;
	local parent = BonusActionBarFrame;
	local frames = self.frames;
	local barInfo = BonusActionBarGetBarInfo(barType);

	for i = 2, #frames do
		obj = frames[i];
		setparent(obj, parent);
		obj:EnableMouse(true);
		obj:ClearAllPoints();
	end

	for i = 3, #frames do
		obj = frames[i];
		if (orientation == "ACROSS") then
			setpoint(obj, "LEFT", frames[i-1], "RIGHT", -3, 0);
		else
			setpoint(obj, "TOP", frames[i-1], "BOTTOM", 0, 22);
		end
	end

	local obj1 = frames[2];  -- CharacterMicroButton
	local obj2 = frames[7];  -- GuildMicroButton

	-- The numbers and points are from BonusActionBarFrame.lua

	setpoint(obj1, "BOTTOMLEFT", barInfo.microBarX or 0, barInfo.microBarY or 0);

	if (barInfo.microTwoRows) then
		setpoint(obj2, "TOPLEFT", obj1, "BOTTOMLEFT", 0, 22);
	end
end

local function addon_UpdateOrientation(self, orientation)
	-- Update the orientation of the frames.
	-- self == menu bar object
	-- orienation == "ACROSS" or "DOWN"

	-- We don't want to rotate the menu bar while it is being shown on
	-- the vehicle frame.
	local hideVehicle = appliedOptions.vehicleHideFrame;
	if (UnitHasVehicleUI("player")) then
		if (not hideVehicle) then
			-- Menu frames are on the vehicle.
			-- Do not rotate them.
			return;
		end
	elseif (not module.actionBarDisabled) then
		local barType = GetBonusBarOverrideBarType() or "default";
		if (barType ~= "default") then
			-- Do not rotate bar when frames are on special bonus action bar frames.
			return;
		end
	end

	-- Menu frames are not on a vehicle.
	-- It is ok to rotate them.
	addon_UpdateOrientationNoVehicle(self, orientation);
end

local function addon_Update(self)
	-- Update the frame
	-- self == menu bar object

	local hideVehicle = appliedOptions.vehicleHideFrame;
	if (UnitHasVehicleUI("player")) then
		if (not hideVehicle) then
			-- Position the menu frames on the vehicle.
			addon_UpdateOrientationInVehicle(self, "ACROSS");
			return;
		end
	elseif (not module.actionBarDisabled) then
		local barType = GetBonusBarOverrideBarType() or "default";
		if (barType ~= "default") then
			addon_UpdateOrientationBonusBar(self, "ACROSS");
			return;
		end
	end

	-- Menu frames are not on a vehicle.
	-- Position the frames where we want them.

	local objChar = CharacterMicroButton;  -- Left most button
	local objHelp = HelpMicroButton;  -- Right most button

	local frame = self.helperFrame;
	frame:ClearAllPoints();
	frame:SetPoint("TOPLEFT", objChar, 0, -20);
	frame:SetPoint("BOTTOMRIGHT", objHelp);

	addon_UpdateOrientationNoVehicle(self, self.orientation);
end

local function addon_Hooked_SetPoint(...)
	-- (hooksecurefunc of each micro button's SetPoint method)
	if (module.ctMenuBar.isDisabled) then
		return;
	end
	addon_Update(module.ctMenuBar);
end

local function addon_Hooked_SetParent(self, parent)
	-- (hooksecurefunc of each micro button's SetParent method)
	if (module.ctMenuBar.isDisabled) then

		-- If the default action bar is disabled...
		if (module.actionBarDisabled) then

			-- Only do this if the menu bar and the default action bar
			-- have both been disabled.

			-- If the game tries to parent the micro buttons to the
			-- BonusActionBarFrame, then it means that the new (WoW 4.0)
			-- larger bonus action bar frame is being displayed.
			--
			-- Because the default action bar has been disabled the
			-- BonusActionBarFrame should not be visible.
			--
			-- We want to disable the mouse in the micro buttons to prevent
			-- the mouse from interfering with anything above the micro
			-- buttons while they are parented to the BonusActionBarFrame.

			if (parent == BonusActionBarFrame) then
				self:EnableMouse(false);
			else
				self:EnableMouse(true);
			end
		end

		return;
	end
	addon_Update(module.ctMenuBar);
end

local function addon_Hooked_UpdateMicroButtons()
	-- (hooksecurefunc of UpdateMicroButtons in MainMenuBarMicroButtons.lua)
	if (module.ctMenuBar.isDisabled) then
		return;
	end
	addon_Update(module.ctMenuBar);
end

local function addon_Hooked_UpdateMicroButtonsParent(parent)
	-- (hooksecurefunc of UpdateMicroButtonsParent in MainMenuBarMicroButtons.lua)
	if (module.ctMenuBar.isDisabled) then
		return;
	end
	addon_Update(module.ctMenuBar);
end

local function addon_Hooked_VehicleMenuBar_MoveMicroButtons(skinName)
	-- (hooksecurefunc of VehicleMenuBar_MoveMicroButtons in VehicleMenuBar.lua)
	module.vehicleSkinName = skinName;
	if (module.ctMenuBar.isDisabled) then
		return;
	end
	addon_Update(module.ctMenuBar);
end

local function addon_Hooked_AnimTransitionFinished(self)
	-- (hooksecurefunc of ActionBar_AnimTransitionFinished in BonusActionBarFrame.lua)

	if (module.ctMenuBar.isDisabled) then
		return;
	end

	--
	-- The original function gets called from BonusActionBarFrame.xml and
	-- MainMenuBar.xml in an <OnFinished> script.
	--
	-- As of WoW 4.0 Blizzard is using special large bonus action
	-- bar frames for some quest items, etc. The frame looks similar
	-- to the vehicle frame, but is just for bonus bar actions.
	--
	-- Blizzard uses an animation to slide the MainMenuBar and
	-- the BonusActionBarFrame in/out. At the end of the animation
	-- they move the exp bar and micro buttons.
	--
	-- Note: If we hide the BonusActionBarFrame using something like a visibility
	-- state driver, the animation "out" finishes, but the next animation "in"
	-- doesn't complete (self.nextAnimBar.slideout:IsPlaying() stays true).
	-- Until it finishes our buttons will not appear in the correct spot.
	-- If we try to move them before the animation finishes then we end up with
	-- them in the wrong spot. Setting the BonusActionBarFrame's alpha to 0
	-- does not cause this issue (although disabling the mouse for the frame doesn't
	-- affect BonusActionBar1, etc).
	--
	-- An alternate approach is to hook UpdateMicroButons() which Blizzard calls
	-- right after repositioning the buttons and before they start to play the
	-- animation. This also prevents the buttons from lingering in sight on the
	-- bars during the animation.
	-- 
	-- Another alternative is to hooksecurefunc each micro button's SetPoint.
	--
	-- self == The bar being animated
	--

	-- Only do the update if these conditions are true.
	-- Doing it at any other time will result in the buttons being positioned
	-- lower on the screen than expected. These two conditions are the only
	-- ones in Blizzard's routine which do not cause an animation group to
	-- start playing.
	if ((self.animOut and not self.nextAnimBar) or (not self.animOut)) then
		addon_Update(module.ctMenuBar);
		module.ctMenuBar:toggleVisibility(2, nil);
	end
end

local function addon_Disable(self)
	if (UnitHasVehicleUI("player")) then
		addon_UpdateOrientationInVehicle(self, "ACROSS");

	elseif (not module.actionBarDisabled) then
		local barType = GetBonusBarOverrideBarType() or "default";
		if (barType ~= "default") then
			addon_UpdateOrientationBonusBar(self, "ACROSS");
		end
	end
end

local function addon_Init(self)
	-- Initialization
	-- self == menu bar object

	appliedOptions = module.appliedOptions;

	module.ctMenuBar = self;

	self.frame:SetFrameLevel(MainMenuBarArtFrame:GetFrameLevel() + 1);

	local frame = CreateFrame("Frame", "CT_BottomBar_" .. self.frameName .. "_GuideFrame");
	self.helperFrame = frame;

	setpoint = frame.SetPoint;
	setparent = frame.SetParent;

	hooksecurefunc("ActionBar_AnimTransitionFinished", addon_Hooked_AnimTransitionFinished);
	hooksecurefunc("UpdateMicroButtons", addon_Hooked_UpdateMicroButtons);
	hooksecurefunc("UpdateMicroButtonsParent", addon_Hooked_UpdateMicroButtonsParent);
	hooksecurefunc("VehicleMenuBar_MoveMicroButtons", addon_Hooked_VehicleMenuBar_MoveMicroButtons);

	return true;
end

local function addon_PostInit(self)
	-- Post initialization
	-- self == menu bar object

	local frames = self.frames;

	for i = 2, #frames do
		hooksecurefunc(frames[i], "SetPoint", addon_Hooked_SetPoint);
		hooksecurefunc(frames[i], "SetParent", addon_Hooked_SetParent);
	end
end

local function addon_Register()
	module:registerAddon(
		"Menu Bar",  -- option name
		"MenuBar",  -- used in frame names
		"Menu Bar",  -- shown in options window & tooltips
		"Menu Bar",  -- title for horizontal orientation
		"Menu",  -- title for vertical orientation
		{ "BOTTOMLEFT", ctRelativeFrame, "BOTTOM", 40, 4 },
		{ -- settings
			orientation = "ACROSS",
			usedOnVehicle = true,
			usedOnBonusBar = true,
		},
		addon_Init,
		addon_PostInit,
		addon_Update,
		addon_UpdateOrientation,
		nil,  -- no enable function
		addon_Disable,
		"helperFrame",
		CharacterMicroButton,
		SpellbookMicroButton,
		TalentMicroButton,
		AchievementMicroButton,
		QuestLogMicroButton,
		GuildMicroButton,
		PVPMicroButton,
		LFDMicroButton,
		MainMenuMicroButton,
		HelpMicroButton
	);
end

module.loadedAddons["Menu Bar"] = addon_Register;
