------------------------------------------------
--               CT_BottomBar                 --
--                                            --
-- Breaks up the main menu bar into pieces,   --
-- allowing you to hide and move the pieces   --
-- independently of each other.               --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

--------------------------------------------
-- Initialization

local _G = getfenv(0);
local module = _G.CT_BottomBar;

-- All of the bar frames are relative to this frame.
module.ctRelativeFrame = UIParent;

local appliedOptions;
local pendingOptions;

local theOptionsFrame;

----------------------------------------------
-- Miscellaneous

local function updateBarWidgets()
	-- Update the widgets in the "Bars" section of the options window.
	local frame = theOptionsFrame;

	if (frame and frame:IsShown()) then

		frame = frame.bar;

		-- Set the enable rep bar checkbox to the same state as the enable exp bar checkbox.
		-- The two options should be synchronized.
		local cbRep = frame["enableReputation Bar"];
		local cbExp = frame["enableExperience Bar"];
		if (cbRep and cbExp) then
			cbRep:SetChecked( cbExp:GetChecked() );
			cbRep:Hide();
			cbRep:Disable();
			cbRep.text:SetTextColor(0.4, 0.4, 0.4, 1);
		end

		-- Set the enable key bar checkbox to the same state as the enable bag bar checkbox.
		-- The two options should be synchronized.
		local cbKey = frame["enableKey Ring"];
		local cbBag = frame["enableBags Bar"];
		if (cbKey and cbBag) then
			cbKey:SetChecked( cbBag:GetChecked() );
			cbKey:Hide();
			cbKey:Disable();
			cbKey.text:SetTextColor(0.4, 0.4, 0.4, 1);
		end

		-- Only show the "Hide" checkbox if the "Enable" one is checked.
		for key, obj in ipairs(module.addons) do
			if (frame["enable" .. obj.optionName]:GetChecked()) then
				frame[obj.optionName]:Show();
			else
				frame[obj.optionName]:Hide();
			end
		end
	end
end

local function updateDisableActionBarWidget()
	local frame = theOptionsFrame;
	if (type(frame) == "table") then
		frame = frame.actionbar;
		local optName = "disableDefaultActionBar";
		local obj = frame[optName];
		obj:SetChecked( appliedOptions[optName] );
	end
end

local preventLoop;
module.updateOptionFromOutside = function(optName, value)
	if (preventLoop) then
		return;
	end

	if (optName == "disableDefaultActionBar") then
		-- Update our option
		module:setOption(optName, value, true);
	end
end

------------------------------------------------
-- Show the drag frames.

local function showModules_CTBarMod(self)
	-- If CT_BarMod is detected then show its bars as well.
	if ( self and CT_BarMod and CT_BarMod.show ) then
		-- Call without self parameter to prevent infinite loop.
		CT_BarMod.show(nil);
	end
end

local function showModules(self)
	-- This will show the drag frames.
	-- It is called when the options window opens.
	module.optionsWindowOpen = true;
	for key, obj in ipairs(module.addons) do
		obj:toggleVisibility(2, nil);
	end
	if (appliedOptions.showCTBarMod) then
		showModules_CTBarMod(self);
	end
end

module.show = showModules;

------------------------------------------------
-- Hide the drag frames.

local function hideModules_CTBarMod(self)
	-- If CT_BarMod is detected then hide its bars as well.
	if ( self and CT_BarMod and CT_BarMod.hide ) then
		-- Call without self parameter to prevent infinite loop.
		CT_BarMod.hide(nil);
	end
end

local function hideModules(self)
	-- This will hide the drag frames.
	-- It is called when the options window closes.
	module.optionsWindowOpen = false;
	for key, obj in ipairs(module.addons) do
		obj:toggleVisibility(2, false);
	end
	hideModules_CTBarMod(self);
end

module.hide = hideModules;

----------------------------------------------
-- Update and apply options.

local function applyUnprotectedOption(optName, value)
	-- Apply an unprotected option.
	--
	-- These will all succeed since they work even if in combat.
	--
	pendingOptions[optName] = nil;
	appliedOptions[optName] = value;
	return true;
end

local function applyProtectedOption(optName, value)
	-- Apply a protected option.
	--
	-- These are options that cannot be applied if we're in combat.
	--
	-- If we can't apply the option, then we'll place the option name
	-- and value in the pendingOptions table. When we get out of combat
	-- we'll apply what ever is in the table.
	--
	if (InCombatLockdown()) then
		pendingOptions[optName] = value;
		return false;
	end
	pendingOptions[optName] = nil;
	appliedOptions[optName] = value;
	return true;
end

function module:applyPendingOptions()
	-- Apply any options that are in the pendingOptions table.
	--
	-- These will only be applied if we are not in combat.
	--
	if (InCombatLockdown()) then
		return;
	end
	for optName, value in pairs(pendingOptions) do
		module:update(optName, value);
	end
end

function module:updateOption(optName, value)
	-- Update an option's value.
	-- optName -- name of option to be changed.
	-- value -- new value for the option.

	if (optName == "petBarScale") then
		value = value or 1;
		if (applyProtectedOption(optName, value)) then
			local obj = module.ctPetBar;
			if (obj and not obj.isDisabled) then
				module.updatePetBar(optName, value);
			end
		end

	elseif (optName == "petBarOpacity") then
		value = value or 1;
		if (applyUnprotectedOption(optName, value)) then
			local obj = module.ctPetBar;
			if (obj and not obj.isDisabled) then
				module.updatePetBar(optName, value);
			end
		end

	elseif (optName == "petBarSpacing") then
		value = value or 6;
		if (applyProtectedOption(optName, value)) then
			local obj = module.ctPetBar;
			if (obj and not obj.isDisabled) then
				module.updatePetBar(optName, value);
			end
		end

	elseif (optName == "vehicleHideFrame") then
		value = not not value;
		if (applyProtectedOption(optName, value)) then
			module.updateVehicleVisibility();
			module.updateBarVisibility();
		end

	elseif (optName == "vehicleHideEnabledBars") then
		value = value ~= false;
		if (applyProtectedOption(optName, value)) then
			module.updateBarVisibility();
		end

	elseif (optName == "vehicleBarAimButtons") then
		value = value ~= false;
		if (applyUnprotectedOption(optName, value)) then
			-- Call the vehicle bar update function
			local obj = module.ctVehicleBar;
			if (obj and not obj.isDisabled) then
				local func = obj.updateFunc;
				if (func) then
					func(obj);
				end
			end
		end

	elseif (optName == "bagsBarHideBags") then
		value = not not value;
		if (applyUnprotectedOption(optName, value)) then
			local obj = module.ctBagsBar;
			obj:update();
		end

	elseif (optName == "repBarHideNoRep" or
		optName == "repBarCoverExpBar" or
		optName == "expBarShowMaxLevelBar"
	) then
		value = not not value;
		if (applyUnprotectedOption(optName, value)) then
			local obj1 = module.ctRepBar;
			local obj2 = module.ctExpBar;
			local update;
			if (obj1 and not obj1.isDisabled) then
				update = true;
			end
			if (obj2 and not obj2.isDisabled) then
				update = true;
			end
			if (update) then
				-- Call Blizzard's function that updates the reputation and exp bars.
				ReputationWatchBar_Update();
			end
		end

	elseif (optName == "hideGryphons") then
		value = value ~= false;
		if (applyUnprotectedOption(optName, value)) then
			module:toggleGryphons(value);
		end

	elseif (optName == "hideTexturesLeft") then
		value = value ~= false;
		if (applyUnprotectedOption(optName, value)) then
			module:hideTexturesLeft(value);
		end

	elseif (optName == "hideTexturesRight") then
		value = value ~= false;
		if (applyUnprotectedOption(optName, value)) then
			module:hideTexturesRight(value);
		end

	elseif (optName == "showCTBarMod") then
		value = not not value;
		if (applyUnprotectedOption(optName, value)) then
			if (value) then
				showModules_CTBarMod(self);
			else
				hideModules_CTBarMod(self);
			end
		end

	elseif (optName == "dragHideTooltip") then
		value = not not value;
		applyUnprotectedOption(optName, value);

	elseif (optName == "clampFrames") then
		value = not not value;
		if (applyUnprotectedOption(optName, value)) then
			for key, obj in ipairs(module.addons) do
				if (not obj.isDisabled) then
					obj:setClamped(value);
				end
			end
		end

	elseif (optName == "disableDefaultActionBar") then
		-- Disable the default main action bar.
		-- Default: true
		value = value ~= false;
		applyUnprotectedOption(optName, value);

		-- Update the checkbox.
		updateDisableActionBarWidget();

		if (CT_BarMod and CT_BarMod.updateOptionFromOutside) then
			-- Set the corresponding option in CT_BarMod.
			-- It is CT_BottomBar that is responsible for the actual
			-- disabling of the default main action bar.
			preventLoop = true;
			CT_BarMod.updateOptionFromOutside(optName, value);
			preventLoop = nil;
		end

	else
		local found;
		for key, obj in ipairs(module.addons) do
			if (optName == obj.optionName) then
				-- Hide or show the bar.
				value = not not value;
				if (not obj.frame:IsProtected()) then
					if (applyUnprotectedOption(optName, value)) then
						obj:toggleVisibility(1);
					end
				else
					if (applyProtectedOption(optName, value)) then
						obj:toggleVisibility(1);
					end
				end
				break;

			elseif (optName == "enable" .. obj.optionName) then
				-- Enable or disable the bar
				updateBarWidgets();
				local value0 = value;
				value = not not value;
				if (not obj.frame:IsProtected()) then

					-- If user enables/disables the exp bar, then do the same
					-- with the rep bar. The two bars work together, and should
					-- both be enabled or disabled at the same time.
					-- The rep bar should be disabled before the exp bar.
					if (optName == "enableExperience Bar") then
						module:setOption("enableReputation Bar", value0, true);
					end

					-- If user enables/disables the bags bar, then do the same
					-- with the key ring. Blizzard attaches the key ring to the bags,
					-- so we'll have both be enabled or disabled at the same time.
					if (optName == "enableBags Bar") then
						module:setOption("enableKey Ring", value0, true);
					end

					if (applyUnprotectedOption(optName, value)) then
						if (value) then
							obj:enable();
						else
							obj:disable();
						end
						-- obj:toggleVisibility(1);
					end
				else
					if (applyProtectedOption(optName, value)) then
						if (value) then
							obj:enable();
						else
							obj:disable();
						end
						-- obj:toggleVisibility(1);
					end
				end
				break;

			end
		end
	end
end

------------------------------------------------
-- Options Frame

local optionsFrameList;

local function optionsInit()
	optionsFrameList = {};

	-- Dummy frame representing a master frame.
	local frame = {};
	frame.offset = 0;
	frame.size = 0;
	frame.details = "";
	frame.yoffset = 0;
	frame.top = 0;
	frame.data = {};

	tinsert(optionsFrameList, frame);
end

local function optionsGetData()
	local frame = optionsFrameList[#optionsFrameList];
	return frame.data;
end

local function optionsAddFrame(offset, size, details, data)
	local yoffset;
	local prevFrame = optionsFrameList[#optionsFrameList];
	if (prevFrame) then
		yoffset = prevFrame.yoffset;
	else
		yoffset = 0;
	end
	yoffset = yoffset + offset;

	local frame = {};
	frame.offset = offset;
	frame.size = size;
	frame.details = details;
	frame.yoffset = 0;
	frame.top = yoffset;
	frame.data = data or {};

	tinsert(optionsFrameList, frame);
end

local function optionsAddObject(offset, size, details)
	local frame = optionsFrameList[#optionsFrameList];
	local yoffset = frame.yoffset + offset;

	details = gsub(details, "%%y", yoffset);
	details = gsub(details, "%%b", yoffset - size);
	details = gsub(details, "%%s", size);
	tinsert(frame.data, details);

	frame.yoffset = yoffset - size;
end

local function optionsAddScript(name, func)
	local frame = optionsFrameList[#optionsFrameList];
	frame.data[name] = func;
end

local function optionsEndFrame()
	local frame = tremove(optionsFrameList);

	local size = frame.size;
	local top = frame.top;
	local bot;
	if (size == 0) then
		bot = top + frame.yoffset;
		size = top - bot + 1;
	else
		bot = top - size - 1;
	end

	local details = frame.details;

	details = gsub(details, "%%y", top);
	details = gsub(details, "%%b", bot);
	details = gsub(details, "%%s", size);

	local prevFrame = optionsFrameList[#optionsFrameList];
	prevFrame.yoffset = bot;
	prevFrame.data[details] = frame.data;
end

local function optionsSetTooltip(self, xoff, yoff, text)
	self:SetScript("OnEnter",
		function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT", xoff, yoff);
			if (type(text) == "table") then
				for key, value in ipairs(text) do
					if (key == 1) then
						GameTooltip:SetText(value);
					else
						GameTooltip:AddLine(value, 1, 1, 1, 1, true);
					end
				end
			else
				GameTooltip:SetText(text);
			end
			GameTooltip:Show();
		end
	);
	self:SetScript("OnLeave",
		function(self)
			GameTooltip:Hide();
		end
	);
end

module.frame = function()
	-- Function to create the options frame.
	-- Once created, module.frame becomes a table reference to the options frame.

	local textColor0 = "1.0:1.0:1.0";
	local textColor1 = "0.9:0.9:0.9";
	local textColor2 = "0.7:0.7:0.7";
	local textColor3 = "0.9:0.72:0.0";
	local offset;

	optionsInit();

	-- Tips
	optionsAddFrame(-5, 0, "frame#tl:0:%y#r");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Tips");
		optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:13:0#r#You can use /ctbb, /ctbottom, or /ctbottombar to open this options window directly.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:13:0#r#Bars are only movable when this options window is open.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:13:0#r#Some options if changed will only be applied when you are not in combat.#" .. textColor2 .. ":l");
	optionsEndFrame();

	-- Options window
	optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Options window");
		optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:dragHideTooltip#Hide drag frame tooltips");
		optionsAddObject(  6,   26, "checkbutton#tl:20:%y#o:clampFrames#Cannot drag bars completely off screen");
		if (CT_BarMod) then
			optionsAddObject( -5,   26, "checkbutton#tl:20:%y#i:showCTBarMod#o:showCTBarMod#Show drag frames for CT_BarMod bars");
		end
	optionsEndFrame();

	optionsAddFrame(-20, 0, "frame#tl:0:%y#r#i:bar");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Bars");

		optionsAddObject(-10, 3*14, "font#t:0:%y#s:0:%s#l:15:0#r#An activated bar can be moved, hidden, and possibly rotated. Some other options may also be available depending on the bar.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 3*14, "font#t:0:%y#s:0:%s#l:15:0#r#When you deactivate a bar, an attempt will be made to restore the bar to its default state if applicable.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 3*14, "font#t:0:%y#s:0:%s#l:15:0#r#If a bar does not deactivate as expected, restart your addons once using the /reload command.#" .. textColor2 .. ":l");

		do
			local off = -15;
			local left, color;
			for key, obj in ipairs(module.addons) do
				if ( obj.frames ) then
					if (obj.optionName == "Reputation Bar" or obj.optionName == "Key Ring") then
						left = 35;
						color = textColor1;
					else
						left = 15;
						color = textColor0;
					end
					optionsAddObject(off,   14, "font#tl:" .. left .. ":%y#" .. obj.addonName .. "#" .. color);
					optionsAddObject( 21,   26, "checkbutton#tl:150:%y" .. "#i:enable" .. obj.optionName .. "#o:enable" .. obj.optionName .. ":true#Activate");
					optionsAddObject( 26,   26, "checkbutton#tl:240:%y" .. "#i:" .. obj.optionName .. "#o:" .. obj.optionName .. ((obj:isDefaultHidden() and ":true") or "") .. "#Hide");
				end
				off = -5;
			end
		end

		optionsAddFrame( -14,   30, "button#t:0:%y#s:200:%s#n:CT_BottomBar_ResetPositions_Button#v:GameMenuButtonTemplate#Reset bar positions");
			optionsAddScript("onclick",
				function(self)
					if ( InCombatLockdown() and self.frame:IsProtected() ) then
						return;
					end
					for key, obj in ipairs(module.addons) do
						obj:resetPosition();
					end
				end
			);
		optionsEndFrame();

		optionsAddScript("onshow",
			function(self)
				for key, obj in ipairs(module.addons) do
					if ( obj.frames ) then
						self["enable" .. obj.optionName]:SetHitRectInsets(0, -40, 0, 0);
						self[obj.optionName]:SetHitRectInsets(0, -30, 0, 0);
					end
				end
				updateBarWidgets();
			end
		);
	optionsEndFrame();

	-- Artwork Options
	optionsAddFrame(-20, 0, "frame#tl:0:%y#r#i:general");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Artwork");

		optionsAddObject( -5,   26, "checkbutton#tl:20:%y#i:hideGryphons#o:hideGryphons:true#Hide the gryphons");
		optionsAddObject(  6,   26, "checkbutton#tl:20:%y#i:hideTexturesLeft#o:hideTexturesLeft:true#Hide left side artwork (main action bar)");
		optionsAddObject(  6,   26, "checkbutton#tl:20:%y#i:hideTexturesRight#o:hideTexturesRight:true#Hide right side artwork (menu, keys, bags)");
	optionsEndFrame();

	-- Vehicle Options
	optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#When You Are In A Vehicle...");

		optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:vehicleHideFrame#Hide the standard vehicle frame.");
		optionsAddObject(  6,   26, "checkbutton#tl:20:%y#o:vehicleHideEnabledBars:true#Hide the activated CT_BottomBar bars.");

		optionsAddObject(  0, 3*14, "font#t:0:%y#s:0:%s#l:50:0#r#The game automatically hides most deactivated bars when you are in a vehicle.#" .. textColor2 .. ":l");
		optionsAddObject(  0, 3*14, "font#t:0:%y#s:0:%s#l:50:0#r#The experience and menu bars will be shown on the vehicle frame if it is not hidden.#" .. textColor2 .. ":l");

	optionsEndFrame();

	-- Action bar options
	optionsAddFrame(-20, 0, "frame#tl:0:%y#r#i:actionbar");
		optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Action Bar");

		optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#CT_BottomBar does not include support for manipulating the default main action bar.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#The main action bar can either be used in its default state or it can be disabled.#" .. textColor2 .. ":l");
		optionsAddObject( -5, 3*14, "font#t:0:%y#s:0:%s#l:20:0#r#If you disable the bar, you may want to use CT_BarMod version 4.004 (or greater) which includes an alternate main action bar.#" .. textColor2 .. ":l");
		optionsAddObject( -2, 3*14, "font#t:0:%y#s:0:%s#l:20:0#r#NOTE: Disabling or enabling the default main action bar will have no effect until addons are reloaded.#" .. textColor3 .. ":l");

		optionsAddObject( -5,   26, "checkbutton#tl:20:%y#i:disableDefaultActionBar#o:disableDefaultActionBar:true#Disable the default main action bar.");

		optionsAddFrame(  -8,   30, "button#t:0:%y#s:180:%s#n:CT_BottomBar_DisableActionBar_Button#v:GameMenuButtonTemplate#Reload addons");
			optionsAddScript("onclick",
				function(self)
					ConsoleExec("RELOADUI");
				end
			);
		optionsEndFrame();
	optionsEndFrame();

	-- Bags Bar
	if (module.ctBagsBar) then
		optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
			optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Bags Bar");

			optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#These options will have no effect if the bags bar is not activated.#" .. textColor3 .. ":l");

			optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:bagsBarHideBags#Hide all buttons except for the backpack");
		optionsEndFrame();
	end

	-- Experience & Reputation Bar
	if (module.ctExpBar and module.ctRepBar) then
		optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
			optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Experience & Reputation Bars");

			optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#These options will have no effect if the experience bar is not activated.#" .. textColor3 .. ":l");

			optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:repBarHideNoRep#Hide rep bar when not monitoring a reputation.");
			optionsAddObject(  6,   26, "checkbutton#tl:20:%y#o:repBarCoverExpBar#Replace the exp bar with the rep bar.");
			optionsAddObject(  6,   26, "checkbutton#tl:20:%y#o:expBarShowMaxLevelBar#Show solid exp bar if no rep and exp bars.");

			optionsAddObject( -2, 5*14, "font#t:0:%y#s:0:%s#l:20:0#r#Enabling all three options emulates the game's behavior for a maximum level character (but works at any level) as long as the 'Hide' options for both the exp bar and the rep bar are unchecked.#" .. textColor2 .. ":l");
			optionsAddObject( -2, 4*14, "font#t:0:%y#s:0:%s#l:20:0#r#If you are using the game's default extra action bars, the game will shift them up or down in response to the showing or hiding of the exp and rep bars.#" .. textColor2 .. ":l");
		optionsEndFrame();
	end

	-- Pet Bar
	if (module.ctPetBar) then
		optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
			optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Pet Bar");

			optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#These options will have no effect if the pet bar is not activated.#" .. textColor3 .. ":l");

			optionsAddFrame( -25,   17, "slider#tl:30:%y#s:250:%s#i:scale#o:petBarScale:1#Scale = <value>#0.25:2:0.01");
			optionsEndFrame();

			optionsAddFrame( -25,   17, "slider#tl:30:%y#s:250:%s#i:opacity#o:petBarOpacity:1#Opacity = <value>#0:1:0.01");
			optionsEndFrame();

			optionsAddFrame( -25,   17, "slider#tl:30:%y#s:250:%s#i:spacing#o:petBarSpacing:6#Button Spacing = <value>#0:25:1");
			optionsEndFrame();
		optionsEndFrame();
	end

	-- Vehicle Tools Bar
	if (module.ctVehicleBar) then
		optionsAddFrame(-30, 0, "frame#tl:0:%y#r");
			optionsAddObject(  0,   17, "font#tl:5:%y#v:GameFontNormalLarge#Vehicle Tools Bar");

			optionsAddObject( -5, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#These options will have no effect if the vehicle tools bar is not activated.#" .. textColor3 .. ":l");
			optionsAddObject( -2, 3*14, "font#t:0:%y#s:0:%s#l:20:0#r#Buttons will only be visible on this bar if you are in a vehicle and you are hiding the standard vehicle frame.#" .. textColor2 .. ":l");
			optionsAddObject( -2, 2*14, "font#t:0:%y#s:0:%s#l:20:0#r#The first spot on this bar will be used for a 'leave vehicle' button when applicable.#" .. textColor2 .. ":l");
			optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:vehicleBarAimButtons:true#Include room for the two aiming buttons.");
		optionsEndFrame();
	end

	-- Reset Options
	optionsAddFrame(-20, 0, "frame#tl:0:%y#r");
		optionsAddObject(  0,    1, "texture#tl:5:%y#br:tr:0:%b#1:1:1");

		optionsAddObject(-15,   17, "font#tl:5:%y#v:GameFontNormalLarge#Reset CT_BottomBar Options");
		optionsAddObject( -5,   26, "checkbutton#tl:20:%y#o:resetAll#Reset options for all of your characters");
		optionsAddFrame( -10,   30, "button#t:0:%y#s:120:%s#v:UIPanelButtonTemplate#Reset options");
			optionsAddScript("onclick",
				function(self)
					if (module:getOption("resetAll")) then
						CT_BottomBarOptions = {};
					else
						if (not CT_BottomBarOptions or not type(CT_BottomBarOptions) == "table") then
							CT_BottomBarOptions = {};
						else
							CT_BottomBarOptions[module:getCharKey()] = nil;
						end
					end
					ConsoleExec("RELOADUI");
				end
			);
		optionsEndFrame();
		optionsAddObject( -2, 4*14, "font#t:0:%y#s:0:%s#l#r#Note: This will reset options and bar positions to default and then reload your UI.#" .. textColor2);
	optionsEndFrame();

	optionsAddScript("onshow",
		function(self)
			showModules(self);
		end
	);

	optionsAddScript("onhide",
		function(self)
			hideModules(self);
		end
	);

	optionsAddScript("onload",
		function(self)
			theOptionsFrame = self;
		end
	);

	return "frame#all", optionsGetData();
end

------------------------------------------------
-- Initialize

function module:optionsInitApplied()
	-- Initialize the applied options table.

	local value, new, old;

	-- We will refer to the applied options table rather than using module:getOption()
	-- during the rest of the addon when we need an option that we know has been
	-- successfully applied.

	appliedOptions.showCTBarMod = not not module:getOption("showCTBarMod");
	appliedOptions.dragHideTooltip = not not module:getOption("dragHideTooltip");
	appliedOptions.clampFrames = not not module:getOption("clampFrames");

	appliedOptions.hideGryphons = module:getOption("hideGryphons") ~= false;
	appliedOptions.hideTexturesLeft = module:getOption("hideTexturesLeft") ~= false;
	appliedOptions.hideTexturesRight = module:getOption("hideTexturesRight") ~= false;

	for key, obj in ipairs(module.addons) do
		local settings = obj.settings;
		local defaultOrientation = (settings.orientation or "ACROSS");

		appliedOptions[obj.optionName] = not not module:getOption(obj.optionName);

		appliedOptions["orientation" .. obj.optionName] = module:getOption("orientation" .. obj.optionName) or defaultOrientation;
		obj.orientation = appliedOptions["orientation" .. obj.optionName];

		appliedOptions["enable" .. obj.optionName] = module:getOption("enable" .. obj.optionName) ~= false;
		obj.isDisabled = not appliedOptions["enable" .. obj.optionName];
	end

	appliedOptions.disableDefaultActionBar = module:getOption("disableDefaultActionBar") ~= false;

	appliedOptions.bagsBarHideBags = not not module:getOption("bagsBarHideBags");

	if (module.ctExpBar and module.ctRepBar) then
		-- Make sure the enable Rep and Exp bar appliedOptions, and the .isDisabled property are synchronized.
		-- The user will be changing the enable exp bar option, which will in turn change the enable rep bar option.
		appliedOptions["enable" .. "Reputation Bar"] = appliedOptions["enable" .. "Experience Bar"];
		module.ctRepBar.isDisabled = module.ctExpBar.isDisabled;
	end

	if (module.ctBagsBar and module.ctKeyRing) then
		-- Make sure the enable Key ring and Bags bar appliedOptions, and the .isDisabled property are synchronized.
		-- The user will be changing the enable bags bar option, which will in turn change the enable key ring option.
		appliedOptions["enable" .. "Key Ring"] = appliedOptions["enable" .. "Bags Bar"];
		module.ctKeyRing.isDisabled = module.ctBagsBar.isDisabled;
	end

	appliedOptions.repBarHideNoRep = not not module:getOption("repBarHideNoRep");
	appliedOptions.repBarCoverExpBar = not not module:getOption("repBarCoverExpBar");
	appliedOptions.expBarShowMaxLevelBar = not not module:getOption("expBarShowMaxLevelBar");

	appliedOptions.petBarScale = module:getOption("petBarScale") or 1;
	appliedOptions.petBarOpacity = module:getOption("petBarOpacity") or 1;
	appliedOptions.petBarSpacing = module:getOption("petBarSpacing") or 6;

	appliedOptions.vehicleBarAimButtons = module:getOption("vehicleBarAimButtons") ~= false;

	appliedOptions.vehicleHideFrame = not not module:getOption("vehicleHideFrame");

	-- Hide enabled bars when in a vehicle.
	old = tonumber(module:getOption("vehicleHideOther"));  -- old option used prior to 4.007
	new = module:getOption("vehicleHideEnabledBars");  -- new option starting with 4.007
	if (new == nil and old ~= nil) then
		-- Convert pre 4.007 option (1 to 3 == hide other bars when in vehicle, 4 == don't hide other bars)
		value = (old ~= 4);
	else
		value = (new ~= false);
	end
	appliedOptions.vehicleHideEnabledBars = value;
end

function module:optionsInit()
	-- Initialize this lua file.
	appliedOptions = module.appliedOptions;
	pendingOptions = module.pendingOptions;
end
